#ifndef STTypes_H
#define STTypes_H

typedef unsigned int    uint;
typedef unsigned long   ulong;

#ifdef _WIN64
    typedef unsigned long long ulongptr;
#else
    typedef ulong ulongptr;
#endif


#define soTouch_ALIGN_POINTER_16(x)      ( ( (ulongptr)(x) + 15 ) & ~(ulongptr)15 )


#if (defined(__GNUC__) && !defined(ANDROID))
#endif


namespace soTouch
{
    #define soTouch_MAX_CHANNELS     16

    #if (defined(__SOFTFP__) && defined(ANDROID))
        // For Android compilation: Force use of Integer samples in case that
        // compilation uses soft-floating point emulation - soft-fp is way too slow
        #undef  soTouch_FLOAT_SAMPLES
        #define soTouch_INTEGER_SAMPLES      1
    #endif

    #if !(soTouch_INTEGER_SAMPLES || soTouch_FLOAT_SAMPLES)
       
        #define soTouch_FLOAT_SAMPLES       1    //< 32bit float samples
     
    #endif

    #if (_M_IX86 || __i386__ || __x86_64__ || _M_X64)

        #define soTouch_ALLOW_X86_OPTIMIZATIONS     1
        #ifdef soTouch_DISABLE_X86_OPTIMIZATIONS
            #undef soTouch_ALLOW_X86_OPTIMIZATIONS
        #endif
    #else
        #undef soTouch_ALLOW_X86_OPTIMIZATIONS

    #endif


    #ifdef soTouch_INTEGER_SAMPLES
        typedef short SAMPLETYPE;
        typedef long  LONG_SAMPLETYPE;

        #ifdef soTouch_FLOAT_SAMPLES
            #error "conflicting sample types defined"
        #endif

        #ifdef soTouch_ALLOW_X86_OPTIMIZATIONS
            #if (!_M_X64)
                #define soTouch_ALLOW_MMX   1
            #endif
        #endif

    #else

        typedef float  SAMPLETYPE;
        typedef float LONG_SAMPLETYPE;

        #ifdef soTouch_ALLOW_X86_OPTIMIZATIONS
            #define soTouch_ALLOW_SSE       1
        #endif

    #endif

    #if ((soTouch_ALLOW_SSE) || (__SSE__) || (soTouch_USE_NEON))
        #if soTouch_ALLOW_NONEXACT_SIMD_OPTIMIZATION
            #define ST_SIMD_AVOID_UNALIGNED
        #endif
    #endif

}

#ifdef ST_NO_EXCEPTION_HANDLING
    #include <assert.h>
    #define ST_THROW_RT_ERROR(x)    {assert((const char *)x);}
#else
    // use c++ standard exceptions
    //#include <stdexcept>
    #include <string.h>
    // #define ST_THROW_RT_ERROR(x)    {throw std::runtime_error(x);}
#endif

#endif
